/****************************************************************************
 Module
   MorseElementsFSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing flat state machines under the 
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 11/04/12 23:55 ddg      create Button.c from TemplateFSM.c
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "MorseElementsFSM.h"
#include "ME218_C32.h"
#include "ES_ServiceHeaders.h"
#include <stdio.h>

/*----------------------------- Module Defines ----------------------------*/


/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/
static  void TestCalibration ( void );
static void CharacterizeSpace ( void );
static void CharacterizePulse( void );
/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static MorseElementsState_t CurrentState;
static unsigned int TimeOfLastRise;
static unsigned int TimeOfLastFall;
static unsigned int LengthOfDot;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;
static unsigned int FirstDelta;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitializeMorseElementsFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority, sets up the initial transition and does any 
     other required initialization for this state machine
 Notes

 Author
     Dustin D. Gerrard 11/5/12
****************************************************************************/
boolean InitializeMorseElements ( uint8_t Priority )
{
  //Declare variables.
  ES_Event NewEvent;
  
  // Initialize the MyPriority variable with the passed in parameter.
  MyPriority = Priority;
     
  // Initialize the port line to receive Morse Code.
  DDRT &= BIT0LO;
  
  // Set CurrentState to be InitMorseElements
   CurrentState = InitMorseElements;
   NewEvent.EventType = ES_INIT;
   PostMorseElementsFSM(NewEvent);
  
  // Set FirstDelta to 0.
  FirstDelta=0;
  
  // End. 
  return True;
}

/****************************************************************************
 Function
     PostTemplateFSM

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
 Notes

 Author
     Dustin D. Gerrard 11/5/12
****************************************************************************/
boolean PostMorseElementsFSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    MorseelementsSM

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   The EventType field of ThisEvent will be one of: RisingEdge, FallingEdge, CalibrationCompleted, EOCDetected, EOWDetected, ButtonDown.  The parameter field of the ThisEvent will be the time that the event occurred.
   Returns True 

 Notes
   uses nested switch/case to implement the machine.
   
 Author
   Dustin D. Gerrard 11/5/12
****************************************************************************/
ES_Event MorseElementsSM( ES_Event ThisEvent )
{  
// Declare variables.
// Local Variables: NextState
ES_Event ReturnEvent;
MorseElementsState_t NextState; 
ReturnEvent.EventType = ES_NO_EVENT;

// Set NextState to CurrentState.
NextState = CurrentState;

// Based on the state of the CurrentState variable choose one of the following blocks of code:
switch ( CurrentState )
{
   case InitMorseElements  :
      if(ThisEvent.EventType == ES_INIT)
      {
      // Set NextState to CalWaitForRise
         NextState = CalWaitForRise;         
      }   
       break;
      
   
   case CalWaitForRise :
     // If ThisEvent is RisingEdge.
     if(ThisEvent.EventType == RisingEdge)
      {
         // Set TimeOfLastRise to ThisTime.
         TimeOfLastRise = ES_Timer_GetTime();
         
         // Set NextState to CalWaitForFall.
         NextState = CalWaitForFall;                
      }
      
      // If ThisEvent is CalibrationComplete.
      if(ThisEvent.EventType == CalibrationComplete)
      {
         // Set NextState to EOC_WaitRise.
         NextState = EOC_WaitRise;
         printf("\n\rLength of Dot: %d", LengthOfDot);
      } 
       break;
   
   
   case CalWaitForFall  :
      // If ThisEvent is FallingEdge.
      if(ThisEvent.EventType == FallingEdge)
      {
         // Set TimeOfLastFall to ThisTime.
         TimeOfLastFall = ES_Timer_GetTime();
         
         // Set NextState to CalWaitForRise
         NextState = CalWaitForRise;
         
         //Call TestCalibration
         TestCalibration();
       }  
       break;
   
   
   case EOC_WaitRise     :
      // If ThisEvent is RisingEdge.
      if(ThisEvent.EventType == RisingEdge)
      {
         // Set TimeOfLastRise to ThisTime.
         TimeOfLastRise = ES_Timer_GetTime();
        
         // Set NextState to EOC_WaitFall 
         NextState =EOC_WaitFall;
         
         // Call CharacterizeSpace function.
         CharacterizeSpace();
      }

      break;
   
   
   case EOC_WaitFall      :
      // If ThisEvent is FallingEdge.
      if(ThisEvent.EventType == FallingEdge)
      {
         // Set TimeOfLastFall to ThisTime.
         TimeOfLastFall = ES_Timer_GetTime();
         
         //Set NextState to EOC_WaitRise.
         NextState = EOC_WaitRise;         
      }
      

      
      // If ThisEvent is EOCDetected.
      if(ThisEvent.EventType == EOC_Detected)
      {
         // Set NextState to DecodeWaitFall.
         NextState = DecodeWaitFall;
      }
      break;
   
   
   case DecodeWaitRise     :
      // If ThisEvent is RisingEdge
      if(ThisEvent.EventType == RisingEdge)
      {
         // Set TimeOfLastRise to ThisTime.
         TimeOfLastRise = ES_Timer_GetTime();
         
         // Set NextState to DecodeWaitFall.
         NextState = DecodeWaitFall;
         
         // Call CharacterizeSpace function.
         CharacterizeSpace();
      }
      
  
      break;
   
   case DecodeWaitFall   :
      // If ThisEvent is FallingEdge.
      if(ThisEvent.EventType == FallingEdge)
      {
         // Set TimeOfLastFall to ThisTime.
         TimeOfLastFall = ES_Timer_GetTime();
         
         // Set NextState to DecodeWaitRise.
         NextState = DecodeWaitRise;
         
         // Call CharacterizePulse function.
         CharacterizePulse();
      }
      

      break;
}
  
  CurrentState = NextState;               
  return ReturnEvent;
} 

/****************************************************************************
 Function
     QueryMorseElementsSM

 Parameters
     None

 Returns
     MorseElementstate_t The current state of the MorseElementsstate machine

 Description
     returns the current state of the MorseElements state machine
 Notes

 Author
     Dustin D. Gerrard 11/5/12
****************************************************************************/
 MorseElementsState_t QueryMorseElementsSM ( void )
{
   return(CurrentState);
}

/***************************************************************************
 private functions
 ***************************************************************************/

 /****************************************************************************
 Function TestCalibration 
 
 notes:
   Takes no parameters, returns nothing.
   
 Author
     Dustin D. Gerrard 11/5/12 

****************************************************************************/
static void TestCalibration ( void )
{
// Declare variables. Local variable SecondDelta.
static unsigned int SecondDelta;
ES_Event NewEvent;

// If calibration is just starting (FirstDelta is 0).
if(FirstDelta == 0)
{
   // Set FirstDelta to most recent pulse width.
   FirstDelta =  TimeOfLastFall - TimeOfLastRise;
} 

else
{  
   // Set SecondDelta to most recent pulse width.
   SecondDelta =  TimeOfLastFall - TimeOfLastRise;
   
   // If (100 * FirstDelta / SecondDelta) less than or equal to 33.
   if((100*FirstDelta/SecondDelta) <=  37 && (100*FirstDelta/SecondDelta) >= 27)
   {
      // Save FirstDelta as LengthOfDot.
      LengthOfDot = FirstDelta;      
      
      // PostEvent CalCompleted.
      NewEvent.EventType = CalibrationComplete;
      PostMorseElementsFSM(NewEvent);      
   } 
   
   // ElseIf (100 * FirstDelta / Second Delta) greater than or equal to 300.
   else if((100*FirstDelta/SecondDelta) >=  250 && (100*FirstDelta/SecondDelta) <= 400)
   {
      // Save SecondDelta as LengthOfDot.
      LengthOfDot = SecondDelta;
      
      // PostEvent CalCompleted.
      NewEvent.EventType = CalibrationComplete;
      PostMorseElementsFSM(NewEvent);            
   } 
   
   // Else (prepare for next pulse).
   else
   {
      // SetFirstDelta to SecondDelta.
      FirstDelta = SecondDelta;
   }
}
}

 /****************************************************************************
 Function CharacterizeSpace
 
 notes: Takes no parameters, returns nothing.
 
 Posts one of EOCDetected Event, EOWDetected Event, BadSpace Event

 Author
     Dustin D. Gerrard 11/5/12
 
****************************************************************************/
static void CharacterizeSpace ( void )
{
// Declare variables. Local variable LastPulseWidth.
static unsigned int LastInterval;
ES_Event NewEvent;

// Calculate LastPulseWidth as TimeOfLastFall - TimeOfLastRise.
LastInterval = TimeOfLastRise - TimeOfLastFall;

printf("\n\rLast Space Interval : %d, DotLength: %d", LastInterval, LengthOfDot);

// If LastPulseWidth OK for a dot.
if((LastInterval >= (3*(LengthOfDot-10))) && (LastInterval <= (3*(LengthOfDot+25))))
{
   // PostEvent DotDetected Event.
//   printf("\n\rEOC Detected.\r\n");
   NewEvent.EventType = EOC_Detected;
   PostMorseElementsFSM(NewEvent);
   PostDecodeMorseService(NewEvent);   
}  

else
{
   // If LastPulseWidth OK for dash.
   if((LastInterval >= (6*(LengthOfDot))) && (LastInterval <= (8*(LengthOfDot))))
   {
      // PostEvent DashDetected Event.
      //printf("\n\rEOW Detected.\r\n");
      NewEvent.EventType = EOW_Detected;
      PostMorseElementsFSM(NewEvent);
      PostDecodeMorseService(NewEvent);
   } 
   
   else
   {
      // PostEvent BadPulse Event.
      NewEvent.EventType = BadSpace;
      PostMorseElementsFSM(NewEvent);
      PostDecodeMorseService(NewEvent);
   }
}
}


 /****************************************************************************
 Function CharacterizePulse
 
 notes: Takes no parameters, returns nothing.
 
 Posts one of DotDetectedEvent, DashDetectedEvent, BadPulseEvent,

 Author
     Dustin D. Gerrard 11/5/12
 
****************************************************************************/
static void CharacterizePulse( void )
{
// Declare variables. Local variable LastPulseWidth.
static unsigned int LastPulseWidth;
ES_Event NewEvent;

// Calculate LastPulseWidth as TimeOfLastFall - TimeOfLastRise.
LastPulseWidth = TimeOfLastFall - TimeOfLastRise;
printf("\n\rLast Pulse Interval : %d, DotLength: %d", LastPulseWidth, LengthOfDot);

// If LastPulseWidth OK for a dot.
if((LastPulseWidth <= (LengthOfDot+10)) && (LastPulseWidth >= (LengthOfDot-10)))
{
   // PostEvent DotDetected Event.
   NewEvent.EventType = DotDetected;
   PostMorseElementsFSM(NewEvent);
   PostDecodeMorseService(NewEvent);
} 
else
{
   // If LastPulseWidth OK for dash.
   if((LastPulseWidth >= 3*(LengthOfDot-10)) && (LastPulseWidth <= 3*(LengthOfDot+25))) 
   {
      // PostEvent DashDetected Event.
      NewEvent.EventType = DashDetected;
      PostMorseElementsFSM(NewEvent);
      PostDecodeMorseService(NewEvent);
   } 
   else
   {
      // PostEvent BadPulse Event.
      NewEvent.EventType = BadPulse;
      PostMorseElementsFSM(NewEvent);
      PostDecodeMorseService(NewEvent);
   }
   
}

return;
}